
#include "db_cint.h"

#include "db.h"
#include "seznam.h"
#include "extra.h"
#include <stdio.h>  // printf
#include "skripty.h"
  /* konkretizace */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* debug prints allowed? */
/*#define DBCINTDBG*/
#define DEBUGSTR(STR) fprintf(stderr, STR)
#define DEBUGSTRNL fprintf(stderr, "\n");

void debug_MR_list(MR_list list);
void MR_list_to_CHandlSeznam(MR_list mrl, CHandlSeznam &outsezn);
void CHandlSeznam_to_MR_list(CHandlSeznam &sezn, MR_list &mrl);
void debug_CHandlSeznam(CHandlSeznam &sezn);


int db_init()
{
    int res;
    const char *dbfile = "./konf_fak.txt";

    res = db_inicializace_fakta(dbfile);

    fprintf(stderr, "Volam db_inicializace_fakta(\"%s\")   ->   %i\n", dbfile, res);
    return res;
}

void db_debugprint()
{
    db_tisk();
}

MR_list db_dohledej(handle BaseHan, MR_list props_in_m)
{
  /* prevedeme seznam z C do C++, zavolame dotaz, prevedeme nalezene z C++ do C */

  CHandlSeznam  props, found;
  MR_list found_in_m;

#ifdef DBCINTDBG
  fprintf(stderr, "db_dohledej: hledam ");debug_MR_list(props_in_m);fprintf(stderr, "\n");
#endif
  
  MR_list_to_CHandlSeznam(props_in_m, props);

#ifdef DBCINTDBG
  fprintf(stderr, "db_dohledej: zadani pro Milana: (%i, [", BaseHan);
  debug_CHandlSeznam(props);
  fprintf(stderr, "])\n");
#endif

  dohledavani_predmetu(BaseHan, props, found);

#ifdef DBCINTDBG
  DEBUGSTR("db_dohledej: Milan vratil toto: ");
  debug_CHandlSeznam(found);
  DEBUGSTRNL;
#endif

  CHandlSeznam_to_MR_list(found, found_in_m);
#ifdef DBCINTDBG
  fprintf(stderr, "db_dohledej: do mercury vracim ");debug_MR_list(found_in_m);fprintf(stderr, "\n");
#endif
  return found_in_m;
}

int db_vyhledej(handle Sense, MR_list in_in_m, MR_list* out_in_m)
{
  /* prevedeme seznam z C do C++, zavolame dotaz, prevedeme nalezene z C++ do C */
  CHandlSeznam  seznam_in_c;
  int res;
  int sense_suffix;

#ifdef DBCINTDBG
  fprintf(stderr, "db_vyhledej: hledam smysl %i: ", Sense);debug_MR_list(in_in_m); fprintf(stderr,"\n");
#endif
  
  MR_list_to_CHandlSeznam(in_in_m, seznam_in_c);

#ifdef DBCINTDBG
 fprintf(stderr, "db_vyhledej: zadani pro Milana: (%i, [", Sense);
  debug_CHandlSeznam(seznam_in_c);
 fprintf(stderr, "])\n");
#endif

  sense_suffix = htokenof(Sense);
  res = db_vyhledavani(&sense_suffix, seznam_in_c);

#ifdef DBCINTDBG
 fprintf(stderr, "db_vyhledej: Milan vratil toto: %i: ", Sense);
  debug_CHandlSeznam(seznam_in_c);
  DEBUGSTRNL;
#endif

  CHandlSeznam_to_MR_list(seznam_in_c, *out_in_m);
#ifdef DBCINTDBG
 fprintf(stderr, "db_vyhledej: do mercury vracim ");debug_MR_list(*out_in_m);fprintf(stderr, "\n");
#endif
  return res;
}


int db_query(int Poradi, handle Sense, MR_list in_in_m, MR_list* out_in_m)
{
  /* prevedeme seznam z C do C++, zavolame dotaz, prevedeme nalezene z C++ do C */
  CHandlSeznam  seznam_in_c;
  int res;
  int sense_suffix;

#ifdef DBCINTDBG
  fprintf(stderr, "db_query: hledam smysl %i: ", Sense);debug_MR_list(in_in_m); fprintf(stderr,"\n");
#endif
  
  MR_list_to_CHandlSeznam(in_in_m, seznam_in_c);

#ifdef DBCINTDBG
 fprintf(stderr, "db_query: zadani pro Milana: (%i, [", Sense);
  debug_CHandlSeznam(seznam_in_c);
 fprintf(stderr, "], poradi=%i)\n", Poradi);
#endif

  sense_suffix = htokenof(Sense);
  res = db_operace(DO_OBSAHUJE, Poradi, sense_suffix, seznam_in_c);

#ifdef DBCINTDBG
 fprintf(stderr, "db_query: Milan vratil toto: (vysledek %i), smysl %i: ", res, Sense);
  debug_CHandlSeznam(seznam_in_c);
  DEBUGSTRNL;
#endif

  CHandlSeznam_to_MR_list(seznam_in_c, *out_in_m);
#ifdef DBCINTDBG
 fprintf(stderr, "db_query: do mercury vracim ");debug_MR_list(*out_in_m);fprintf(stderr, "\n");
#endif
  return res;
}

int db_set(handle Sense, MR_list in_in_m)
{
  /* prevedeme seznam z C do C++, zavolame dotaz, prevedeme nalezene z C++ do C */
  CHandlSeznam  seznam_in_c;
  int res;
  int sense_suffix;

#ifdef DBCINTDBG
  fprintf(stderr, "db_set: hledam smysl %i: ", Sense);debug_MR_list(in_in_m); fprintf(stderr,"\n");
#endif
  
  MR_list_to_CHandlSeznam(in_in_m, seznam_in_c);

#ifdef DBCINTDBG
 fprintf(stderr, "db_set: zadani pro Milana: (%i, [", Sense);
  debug_CHandlSeznam(seznam_in_c);
 fprintf(stderr, "], poradi=%i)\n", Poradi);
#endif

  sense_suffix = htokenof(Sense);
  res = db_operace(DO_ZAPIS, 0, sense_suffix, seznam_in_c);

#ifdef DBCINTDBG
 fprintf(stderr, "db_set: Milan vratil toto: (vysledek %i), smysl %i: ", res, Sense);
  debug_CHandlSeznam(seznam_in_c);
  DEBUGSTRNL;
#endif

  return res;
}

/******** Konkretizace ***********/

void db_reset_konkretizace()
{
  set_db_pos(0);
}

int db_vyrob_dalsi_konkretni(member in, member* out)
{
  Member *tempout = new Member;
  
  if (konkretizuj_dalsi((Member*)in, tempout,19005440)) {
      fprintf(stderr, "db_vyrob_dalsi_konkretni: %lu: ", tempout);
      member_print((member)tempout);
      fprintf(stderr, "\n");
      *out = (member)tempout;
      return 1;
  } else {
      delete tempout;
      *out = 0;
      return 0;
  }
}


/******** Konverzni funkce **************/

/* -- seznam intu z Mercury do CHandlSeznam a naopak */

void MR_list_to_CHandlSeznam(MR_list mrl, CHandlSeznam &outsezn)
{
  while (! MR_list_is_empty(mrl) ) {
    outsezn.pridej_na_konec(MR_list_head(mrl));
    mrl = MR_list_tail(mrl);
  }
}

void CHandlSeznam_to_MR_list(CHandlSeznam &sezn, MR_list &mrl)
{
  MR_list revmrl;
/* fprintf(stderr, "CHandlSeznam_to_MR_list: called\n");*/
  revmrl = my_MR_list_nil();
/* fprintf(stderr, "CHandlSeznam_to_MR_list: revmrl=%i\n", revmrl);*/
  sezn.nastav_zacatek_cteni();
  while (! sezn.konec_cteni()) {
    revmrl = my_MR_list_cons( sezn.precti_info(), revmrl );
/*   fprintf(stderr, "CHandlSeznam_to_MR_list: revmrl=%i\n", revmrl);*/
  }
/* fprintf(stderr, "CHandlSeznam_to_MR_list: revmrl finished=%i\n", revmrl);*/
/*  debug_MR_list(revmrl);fprintf(stderr, "\n");*/
/* fprintf(stderr, "CHandlSeznam_to_MR_list: reverting list.\n");*/
  mrl = MR_list_rev(revmrl);
/* fprintf(stderr, "CHandlSeznam_to_MR_list: returning: %i\n", mrl);*/
}


/* Konstruktor a destruktor CVety */

void setAttrs_CVeta(CVeta &veta, handle mluvci, handle posluchac, handle mood) {
    veta.Mluvci = mluvci;
    veta.Posluchac = posluchac;
    switch(mood) {
        case 2704015360u:  veta.TypVety = TV_OTAZKA; break;
        case 2703949824u: veta.TypVety = TV_PRIKAZ; break;
        case 2703884288u:  veta.TypVety = TV_OZNAMENI; break;
    }
}

void cons_CVeta(CVeta &veta, handle mluvci, handle posluchac, handle mood, MR_list sense) {
    MR_list_to_CHandlSeznam(sense, veta);
    /* prevedl jsem seznam handlu do vety */
    veta.SetAttrs(mluvci, posluchac, mood);
}


void decons_CVeta(CVeta &veta, handle &mluvci, handle &posluchac, handle &mood, MR_list &sense) {
    CHandlSeznam_to_MR_list(veta, sense);
    veta.GetAttrs(mluvci, posluchac, mood);
}


/* Ladici vypis Mercuryovskeho i Milanova seznamu intu */

void debug_CHandlSeznam(CHandlSeznam &sezn)
{
  sezn.nastav_zacatek_cteni();
  while (! sezn.konec_cteni()) {
     fprintf(stderr, "%i ", (int)sezn.precti_info());
  }
}

void debug_MR_list(MR_list list) {
    if (MR_list_is_empty(list)) {
       fprintf(stderr, "[]");
    } else {
       fprintf(stderr, "[");
       fprintf(stderr, "%ld", (long) list_head(list));
        list = MR_list_tail(list);
        while (!list_is_empty(list)) {
           fprintf(stderr, ", %ld", (long) MR_list_head(list));
            list = MR_list_tail(list);
        }
       fprintf(stderr, "]");
    }
}

#ifdef __cplusplus
}
#endif /* __cplusplus */

