:- module db_mint.
%%% Modul pro napojeni na milanovu databazi.

:- interface.

:- import_module int, list, member.
:- import_module fstype.

:- type dbstate ---> dbstate(int).

:- type db --->
      db(dbstate::dbstate).

:- func db_mint__init = (db::out) is det.
%% volani Milanovy inicializace
:- func db_mint__init(int::in) = (db::out) is det.
%% Milanovu inicializaci uz volal nekdo jiny, ted nam jen rika jeji vysledek.
:- func db_mint__fake_db_handle = (db::out) is det.
%% vraci handle databaze, jako by byla uspesne inicializovana, "z globalni promenne"

:- pred db_mint__ok(db::in) is semidet.

:- pred db_mint__cc_dbq(int::in, list(int)::in, list(int)::out) is semidet.
:- pred db_mint__cc_dbq(db::in, int::in, list(int)::in, list(int)::out) is semidet.
:- pred db_mint__dbq(int::in, list(int)::in, list(int)::out) is nondet.
:- pred db_mint__dbq(db::in, int::in, list(int)::in, list(int)::out) is nondet.
% provadi dboperaci vyhledavani. O spravny tvar seznamu handlu se musi starat volajici. Neuspech
% pro nenalezeni se nelisi od neuspechu pro spatne volani.

:- pred db_mint__set(int::in, list(int)::in) is det.
:- pred db_mint__set(db::in, int::in, list(int)::in) is det.

:- pred db_mint__vyhledej(int::in, list(int)::in, list(int)::out) is semidet.
:- pred db_mint__vyhledej(db::in, int::in, list(int)::in, list(int)::out) is semidet.
%% volani vyhledavani v databazi. Na vstupu je handle smyslu
% a pripadne seznam omezujicich parametru
% Na vstupu pak seznam parametru tak, jak byly co nejverohodneji doplneny
% Selze, pokud nenajde, nebo nastala chyba.

:- pred db_mint__dohledej(int::in, list(int)::in, list(int)::out) is semidet.
:- pred db_mint__dohledej(db::in, int::in, list(int)::in, list(int)::out) is semidet.
%% volani milanova dohledavatka v databazi

:- pred db_mint__konkretizuj_vse(db::in, member::member_ui, list(fs)::out) is det.
%% Najde v DB vse, co odpovida danemu memberu.
%% !!! Member NEdealokuje !!! Vysledky naopak ALOKUJE
%% Mode by tedy mel byt member::ui, list(member)::uo, ale ono to v Mercury jeste nefunguje

:- pred db_mint__debugprint(db::in) is det.
%% volani milanova dohledavatka v databazi

:- implementation.

:- import_module handle.
:- import_module debugstr.
:- import_module std_util. % solutions
:- import_module list_tools. % get_last


ok(db(dbstate(0))).
% Jen inicializovana databaze je v poradku.


% Export predikatu je proto, aby db_mint generoval db_mint.h, ktery potrebuji includovat.
% Jediny duvod k inkludovani je typ MR_Word.
:- pred dummy_pred is failure.
:- pragma export(dummy_pred, "c_dummy_pred").
dummy_pred :- fail.

%%% Pomocne funkce, aby C++ dokazalo vyrabet MR_list:
%

:- func nil = list(int).
:- func cons(int, list(int)) = list(int).
:- func rev(list(int)) = list(int).

nil = [].
cons(X, Y) = [X|Y].
rev(X) = list__reverse(X).

:- pragma export(nil = out, "my_MR_list_nil").
:- pragma export(cons(in, in) = out, "my_MR_list_cons").
:- pragma export(rev(in)= out, "MR_list_rev").


%
%%% Konec pomocnych funkci


:- pragma c_header_code("#include ""db_cint.h""").


init = Out :-
  Status = c_init,
  Out = db(dbstate(Status)).

init(Status) = db(dbstate(Status)).

fake_db_handle = db(dbstate(0)).

dohledej(Han, InList, OutList) :-
  c_dohledej(Han, InList, OutList).
dohledej(DB, Han, InList, OutList) :-
  ok(DB),
  c_dohledej(Han, InList, OutList).

:- import_module require. % error
vyhledej(Sense, InList, OutList) :-
  vyhledej(fake_db_handle, Sense, InList, OutList).
vyhledej(DB, Sense, InList, OutList) :-
  ok(DB),
  c_vyhledej(Sense, InList, OutList, Result),
  (
  if Result = 0
  then fail % nenasel
  else
    if Result = -1
    then error("db_mint__vyhledej: Chyba pri vyhledavani v databazi")
    else true
  ).
  
dbq(Sense, InList, OutList) :-
  dbq(fake_db_handle, Sense, InList, OutList).
dbq(DB, Sense, InList, OutList) :-
  if not ok(DB)
  then error("db_mint__dbq: Neni inicializovana databaze")
  else
    % debugstr("db_mint__dbq: c_dbq ptam se Milana: ", {Sense, [handle__any, handle__any, handle__any, handle__any | InList]}),
    c_dbq(Sense, [handle__any, handle__any, handle__any, handle__any | InList],
                 [_, _, _, _ | OutList], Result),
    % debugstr("db_mint__dbq: c_dbq returned: ", {OutList, Result}),
    (
    if Result = 0
    then fail % nenasel
    else
      if Result = -1
      then error("db_mint__dbq: Chyba pri vyhledavani v databazi")
      else true
    )
  .

set(Sense, InList) :-
  set(fake_db_handle, Sense, InList).
set(DB, Sense, InList) :-
  if not ok(DB)
  then error("db_mint__set: Neni inicializovana databaze")
  else
    c_set(Sense, [handle__any, handle__any, handle__any, handle__any | InList],
                 Result),
    (
    if Result = -1
    then error("db_mint__set: Chyba pri vkladani do databaze.")
    else true
    )
  .

cc_dbq(Sense, InList, OutList) :-
  OutList = promise_only_solution(
    pred(Out::out) is cc_nondet :- dbq(fake_db_handle, Sense, InList, Out)
  ).
cc_dbq(DB, Sense, InList, OutList) :-
  OutList = promise_only_solution(
    pred(Out::out) is cc_nondet :- dbq(DB, Sense, InList, Out)
  ).

/*
konkretizuj_vse(DB, Needle, Found) :-
  if not ok(DB)
  then error("db_mint__konkretizuj_vse: Neni inicializovana databaze.")
  else
    solutions(c_konkretizuj(Needle), FoundWithGhost),
    (
    if list_tools__get_last(FoundWithGhost, TFound, _Ghost)
    then Found = TFound
    else Found = FoundWithGhost
    ),
    debugstr("db_mint__konkretizuj_vse: celkem nasel membery: ", Found),
*/    /*
    list__filter(
      (pred(M::in) is semidet:-
         debugstr("db_mint__konkretizuj_vse: nasel: "), member__print(M), debugstr("\n"),
         semidet_succeed
      ), Found, _),
    */
/*    debugstr("db_mint__konkretizuj_vse: konec konkretizace\n")
  .
*/
:- import_module fs.
:- import_module bool.

konkretizuj_vse(DB, Needle, Found) :-
  if not ok(DB)
  then error("db_mint__konkretizuj_vse: Neni inicializovana databaze.")
  else
    solutions(
      (pred(FS::out) is nondet:-
        c_konkretizuj(Needle, FoundMember, DidFound),
        DidFound = yes,
        % debugstr("konkretizuj_vse: nasel zase jeden: ", FoundMember),
        member__to_fs(FoundMember, FS),
        % debugstr("konkretizuj_vse: udelal z nej fs: ", fs__to_string(FS))
        true
      ), Found),
    % debugstr("db_mint__konkretizuj_vse: celkem nasel fs: ", Found),
    % debugstr("db_mint__konkretizuj_vse: konec konkretizace\n")
    true
  .

:- func c_init = (int::out) is det.
:- pred c_dbq(int::in, list(int)::in, list(int)::out, int::out) is multi.
:- pred c_set(int::in, list(int)::in, int::out) is det.
:- pred c_dohledej(int::in, list(int)::in, list(int)::out) is det.
:- pred c_vyhledej(int::in, list(int)::in, list(int)::out, int::out) is det.

:- pred c_konkretizuj(member::member_ui, member::member_uo, bool::out) is nondet.
% Najde dalsi mozne reseni, vstupni member NERUSI!
% Navic kvuli chybe v Mercury si jeste sdeluji priznak, jestli jsem opravdu jako
% neco nasel.

:- pragma c_code(c_init = (Status::out), [will_not_call_mercury],
      "
      Status = db_init(); 
      ").

:- pragma c_code(c_dohledej(BaseHan::in, PropList::in, ObjList::out), [may_call_mercury],
      "
      ObjList = db_dohledej(BaseHan, PropList);
      ").

:- pragma c_code(c_vyhledej(Sense::in, InList::in, OutList::out, Res::out), [may_call_mercury],
      "
      Res = db_vyhledej(Sense, InList, &OutList);
      ").

:- pragma c_code(debugprint(_Dummy::in), [may_call_mercury],
      "
      db_debugprint();
      ").


:- pragma c_code(c_dbq(Sense::in, InList::in, OutList::out, Res::out), [may_call_mercury],
        local_vars("
                int poradi;
                int sense;
                MR_list inlist;
        "),
        first_code("
                LOCALS->poradi = 0;
                LOCALS->sense = Sense;
                LOCALS->inlist = InList;
        "),
        % Pocitadlo Milan zvysuje sam
        retry_code("
        "),
        common_code("
                LOCALS->poradi = db_query(LOCALS->poradi, LOCALS->sense, LOCALS->inlist, &OutList);
                Res = LOCALS->poradi;
                if (Res == 0) { FAIL; } else { SUCCEED; };
        ")
).

:- pragma c_code(c_set(Sense::in, InList::in, Res::out), [may_call_mercury],
      "
      Res = db_set(Sense, InList);
      ").
      

:- pragma c_code(c_konkretizuj(Needle::member_ui, Found::member_uo, DidFound::out), [may_call_mercury],
        local_vars("
                member Needle2;
        "),
        first_code("
                db_reset_konkretizace();
                LOCALS->Needle2 = Needle;
        "),
        retry_code("
                fprintf(stderr, ""c_konkretizuj: dalsi pokus\n"");
        "),
        common_code("
                if (db_vyrob_dalsi_konkretni(LOCALS->Needle2, &Found)) { 
                      /* fprintf(stderr, ""c_konkretizuj: uspech %lu\n"", Found); */
                      DidFound = 1;
                      SUCCEED;
                } else {
                      /* fprintf(stderr, ""c_konkretizuj: dalsi neni.\n""); */
                      Found = 0;
                      DidFound = 0;
                      FAIL;
                }
        ")
).

