:- module member.
% Modul zavadi do Mercury moznost pracovat s membery.
% Spoleha na member_cint, ze bude obsahovat konstruktory a destruktory memberu.
:- interface.

:- import_module fstype.
:- import_module handle.
:- import_module bool.
:- import_module string.

:- type member.

:- inst member(I) = bound(member(I)).
:- inst member == member(ground).
:- inst member_skel == member(free).

        % XXX the current Mercury compiler doesn't support `ui' modes,
        % so to work-around that problem, we currently don't use
        % unique modes in this module.

% :- inst uniq_member = unique(member).
% :- inst uniq_member == uniq_member(unique).
:- inst uniq_member(I) = bound(member(I)). % XXX work-around
:- inst uniq_member == uniq_member(ground). % XXX work-around
:- inst uniq_member_skel == uniq_member(free).

:- mode member_di == di(uniq_member).
:- mode member_uo == out(uniq_member).
:- mode member_ui == in(uniq_member).


:- func member__blank = (member::member_uo) is det.

:- pred member__new_from_handle(handle::in, member::member_uo) is det.

:- pred member__set(handle::in, handle::in, bool::in, member::member_di, member::member_uo) is det.
% Nastavi dany sloupec memberu na danou hodnotu s danym priznakem negace.

:- pred member__set(handle::in, handle::in, member::member_di, member::member_uo) is det.
% Nastavi dany sloupec memberu na danou hodnotu bez priznaku negace.

:- pred member__get(member::member_ui, handle::in, handle::out, bool::out) is det.
:- pred member__get(member::member_ui, handle::in, handle::out) is det.
% Zjisti, jakou hodnotu a priznak negace ma dany sloupec memberu.

:- pred member__free(member::member_di) is det.
% dealokuje member a poznamena, ze je dealokovan

:- pred member__print(member::member_ui) is det.
% Vypise member na stderr.


%%%%%%%%%% Prevod fs na member a naopak

:- pred member__from_fs(fs::in, member::member_uo) is det.
% vse, co se z fs da vyjadrit v memberu tak vyjadri
% XXX Neresi problem fs, ktera je viceznacna! Vezme prvni mozne cteni a to 
% do memberu ulozi.

:- pred member__to_fs(member::member_di, fs::out) is det.
% vytvori fs, ktera ponese vse, co vyjadruje member a lze ve fs vyjadrit

% :- pred members_to_fss(list(member)::member_di, list(fs)::out) is det.



:- implementation.

:- import_module stringutils.
:- import_module require.
:- import_module debugstr.

:- type member --->
        member(c_pointer).

:- pragma c_header_code("#include ""member_cint.h""").

:- pragma c_code(member__blank = (Member::member_uo),
        [will_not_call_mercury],
        "Member = member_blank();").

member__new_from_handle(Han, OutMember) :-
  member__set(0x10000000, Han, member__blank, OutMember).


:- pred member__semidet_set(handle::in, handle::in, bool::in, member::member_di, member::member_uo) is semidet.

:- pragma c_code(member__semidet_set(Col::in, Val::in, Neg::in, InMember::member_di, OutMember::member_uo),
        [will_not_call_mercury],
        "if (InMember) {
                member_set(Col, Val, Neg, InMember, &OutMember);
                SUCCESS_INDICATOR=1;
         } else
                SUCCESS_INDICATOR=0;
        ").

member__set(Col, Val, Neg, In, Out) :-
  if semidet_set(Col, Val, Neg, In, TOut)
  then Out = TOut
  else error("member__set: Failed to set column "++dump(Col)++" to "++dump(Val)++", member not allocated!").

member__set(Col, Val) -->
  member__set(Col, Val, yes).

:- pred member__semidet_get(member::member_ui, handle::in, handle::out, bool::out) is det.

:- pragma c_code(member__semidet_get(Member::member_ui, Col::in, Val::out, Neg::out),
        [will_not_call_mercury],
        "
         member_get(Member, Col, &Val, &Neg);
        ").

member__get(Member, Col, Val, Neg) :-
  % debugstr("member__get: Mame Member: ", Member),
  % debugstr("member__get: Vytiskneme ho: "), member__print(Member), debugstr("\n"),
  (
  if semidet_get(Member, Col, TVal, TNeg)
  then Val = TVal, Neg = TNeg
  else error("member__get: Failed to get column "++dump(Col)++", member not allocated!")
  ).

member__get(Member, Col, Val) :-
  member__get(Member, Col, Val, _Neg).


:- pragma c_code(free(InMember::member_di),
        [will_not_call_mercury],
        "member_free(InMember);").

:- pragma c_code(print(InMember::member_ui),
        [will_not_call_mercury],
        "member_print(InMember);").


%%%%%%%%%% Prevod fs na member a naopak

:- import_module std_util.
:- import_module list.
:- import_module fs.

member__from_fs(da(Atom), _) :- error("member__from_fs: Unsupported call fro datatom "++dump(Atom)).
member__from_fs(or(ORL), _) :- error("member__from_fs: Unsupported call for ORList "++dump(ORL)).
member__from_fs(fs(FSPairs), Member) :-
  list__foldl(
    (pred((FSName-FSVal)::in, IM::in, OM::out) is det:-
      if FSVal = da(FSValAtom), col_fspair(GotCol, GotVal, FSName, FSValAtom)
      then
        member__set(GotCol, GotVal, no, IM, OM)
      else
        IM = OM
    ), FSPairs, member__blank, TMember),
  (
  if cc_extract(fs(FSPairs), han-ape, da(han(Han)))
  then member__set(0x10000000, Han, no, TMember, Member)
  else
    if cc_extract(fs(FSPairs), typ-ape, da(han(Han)))
    then member__set(0x10000000, Han, no, TMember, Member)
    else Member = TMember
  ).
  

member__to_fs(Member, FS) :-
  solutions((pred(C::out) is multi:-col(C)), Cols),
  % debugstr("member__to_fs: called, will scan cols: ", Cols),
  list__foldl(
    (pred(Col::in, I::in, O::out) is det:-
      member__get(Member, Col, GotVal, GotNeg),
      % debugstr("member__to_fs: got: ", {Col, GotVal, GotNeg}),
      (if GotNeg = yes then error("member__to_fs: Negation flag not supported in fs for column "++dump(Col)++", value "++dump(GotVal)) else true),
      (
      if GotVal = handle__any
      then O = I
      else
        if col_fspair(Col, GotVal, Name, FSVal)
        then
          (
          if unify(I, fs([Name-da(FSVal)]), TO)
          then O=TO
          else error("member__to_fs: Internal error, failed to unify "++dump(I)++" with "++dump(Name-FSVal))
          )
        else
          error("member__to_fs: Internal inconsistency, column "++dump(Col)++" is not expressed in fs!")
      )
    ), Cols, fs__blank, TFS),
  member__get(Member, 0x10000000, Han, GotNeg2),
  (if GotNeg2 = yes then error("member__to_fs: Negation flag not supported in fs for column "++dump(0x10000000)++", value "++dump(Han)) else true),
  (
  if Han = handle__any
  then FS = TFS
  else
    (
    if Han = striptoken(Han)
    then
      UFS = fs([typ-da(han(Han))])
    else
      UFS = sort(fs([typ-da(han(striptoken(Han))), han-da(han(Han))]))
    ),
    (
    if unify(TFS, UFS, T2FS)
    then FS = T2FS
    else error("member__to_fs: Internal error, failed to unify "++dump(TFS)++" with "++dump(UFS))
    )
  ),
  % debugstr("member__to_fs: nyni nealokuji member: "), print(Member), debugstr("\n"),
  member__free(Member).



% members_to_fss([], []).
% members_to_fss([M|Tail], [F|FTail]) :-
  % member__to_fs(M, F),
  % members_to_fss(Tail, FTail).


% Pouziva se pro prevod memberu na fs, tyto sloupce budou prohledavany a prevadeny.
:- pred col(handle).
:- mode col(out) is multi.

% Takto budou ty sloupce prevadeny.
:- pred col_fspair(handle, handle, dagname, dagatom).
:- mode col_fspair(in, in, out, out) is semidet.
:- mode col_fspair(out, out, in, in) is semidet.

% lingv. nezajimave:
% predmet
% priorita
% relevance
% skupina
% col(0x011b0000). % lingvisticky je pristup nezajimavy
% col_fspair(0x011b0000, X, prop(0x011b0000), prop(X)).

col(0x80080000).
col_fspair(0x80080000, X, prop(0x80080000), prop(X)).

col(0x70000000).  % mistnost
col_fspair(0x70000000, X, room, han(X)).

col(0x01180000).  % ... kontejner/ent/dlazdice
col_fspair(0x01180000, X, container, han(X)).
col(0x01190000).  % kde  ... v/na kontejneru
col_fspair(0x01190000, X, container_inon, han(X)).

col(0x80160000).
col_fspair(0x80160000, X, prop(0x80160000), prop(X)).

col(0x802d0000).
col_fspair(0x802d0000, X, prop(0x802d0000), prop(X)).

col(0x802e0000).
col_fspair(0x802e0000, X, prop(0x802e0000), prop(X)).

col(0x80140000).
col_fspair(0x80140000, X, prop(0x80140000), prop(X)).

col(0x80280000).
col_fspair(0x80280000, X, prop(0x80280000), prop(X)).

col(0x803f0000).
col_fspair(0x803f0000, X, prop(0x803f0000), prop(X)).

col(0x011c0000).
col_fspair(0x011c0000, X, prop(0x011c0000), prop(X)).

col(0x011d0000).
col_fspair(0x011d0000, X, prop(0x011d0000), prop(X)).

col(0x011e0000).
col_fspair(0x011e0000, X, prop(0x011e0000), prop(X)).

col(0x01140000).
col_fspair(0x01140000, X, count, FSVal) :-
  FSVal = int(Int),
  X = int2han(Int).

/* Outdated:
% V primem smeru pouziti prevadi handle dlazdice (zaokrouhlenim) na handle
% mistnosti, handly kontejneru nechava beze zmeny
% V opacnem smeru necha handle beze zmeny uplne.
:- func trim_exact_loc(handle) = handle.
:- mode trim_exact_loc(in) = out is det.
:- mode trim_exact_loc(out) = in is det.
:- pragma promise_pure(trim_exact_loc/1).

trim_exact_loc(In::in) = (Out::out) :-
  if mtypeof(In) = mtypeof(0x70000000)
  then % urizni token
    Out = striptoken(In)
  else % nech kontejner beze zmeny
    Out = In.

trim_exact_loc(Out::out) = (Out::in).
*/

