/********************************************************
* zdrojovy soubor: SEZNAM.CPP
* autor: Milan Hladik
* verse: 1.3.
* datum: 24.5.2002
*********************************************************
* obsah: 
*   - definice tridy CHandlSeznam, representujici spojovy seznam
*     s prvky typu THandl
*   - obsahuje metody pro operace se spojovym seznamem:
*     pridavani (na zacatek, konec, pripojeni), vymazani,
*     cteni (postupne, z libovolne posice),... 
*********************************************************/

#include "seznam.h"
#include "db.h"

/********************************************************
*  definice metod tridy CHandlSeznam
*********************************************************/


/** implicitni konstruktor */

CHandlSeznam::CHandlSeznam(){	

  Head=0;
  End=&Head;
  PostupneCteni=Head;
}


/** kopirovaci konstruktor */

CHandlSeznam::CHandlSeznam(const CHandlSeznam& NovySeznam){	

  Head=fill(NovySeznam.Head); ///zkopiruje vlastni seznam
  nastav_posici_end(); ///nastavi ukazatel End na konec seznamu

}


/** destruktor */

CHandlSeznam::~CHandlSeznam(){	
  destroy(Head);
}


/** operator prirazeni */

const CHandlSeznam& CHandlSeznam::operator=(const CHandlSeznam& NovySeznam){	

  destroy(Head); ///smaze stary seznam
  Head=fill(NovySeznam.Head); ///zkopiruje vlastni seznam
  nastav_posici_end(); ///nastavi ukazatel End na konec seznamu

  return *this;
}



void CHandlSeznam::pridej_na_zacatek(THandl NoveInfo){
/** prida novy clen seznamu s hodnotou NoveInfo na zacatek */
	
TPrvekHandlSeznamu* NovyPrvek;

  if(!(NovyPrvek=new TPrvekHandlSeznamu)) m_error();
  NovyPrvek->Info=NoveInfo;
  NovyPrvek->Next=Head;
  Head=NovyPrvek;

  /* jedna-li se o pridani prvniho prvku do CHandlSeznamu,
     musime poopravit ukazatel na posledni prvek */
		
  if(*End==Head) End=&(Head->Next);	
 
} ///of pridej_na_zacatek()


			
void CHandlSeznam::pridej_na_konec(THandl NoveInfo){
/** prida novy clen seznamu s hodnotou NoveInfo na konec */
	
TPrvekHandlSeznamu* NovyPrvek; ///pridavany prvek

  if(!(NovyPrvek=new TPrvekHandlSeznamu)) m_error();
  NovyPrvek->Info=NoveInfo;
  NovyPrvek->Next=0;
  (*End)=NovyPrvek;

  End=&((*End)->Next);	//poopravime ukazatel na posledni prvek
}



int CHandlSeznam::precti_arg(int CisloPoradi, THandl* OutputArg) {
/** vyznam: precte hodnotu clenu seznamu s poradovym cislem CisloPoradi
            a vrati ji v parametru OutputArg
    vystup: 1 pri uspechu,
            0 jinak (zada se vyssi poradove cislo, nez je delka seznamu) */

TPrvekHandlSeznamu* pAkt=Head; /* ukazatel na aktualni prvek,
                                  prochazime jim seznam */
int i=1; ///poradove cislo

  if(CisloPoradi<1) return 0; ///chybna hodnota parametru CisloPoradi

  while(pAkt&&(i<CisloPoradi)) {
    pAkt=pAkt->Next;
    i++;
    }

  if(!pAkt) 
    ///chteli bysme argument s poradivym cislem vestsim nez je delka seznamu
    return 0; 

  else { ///vse je OK
    *OutputArg=pAkt->Info;
    return 1;
    }

} ///of nastav_arg()




int CHandlSeznam::nastav_arg(int CisloPoradi, THandl InputArg) {
/** vyznam: nastavi hodnotu clenu seznamu s poradovym cislem CisloPoradi
            na hodnotu InputArg; je-li CisloPoradi>delka seznamu, doplni
            zbyle hodnoty na VOLNA_PROMENNA
    vystup: 1 pri uspechu,
            0 jinak (zada-li se poradove cislo zaporne ci 0) */

TPrvekHandlSeznamu** pAkt=&Head; /* ukazatel na aktualni prvek,
                                  prochazime jim seznam */
int i=1; ///poradove cislo

  if(CisloPoradi<1) return 0; ///chybna hodnota parametru CisloPoradi

  while((*pAkt) && (i<CisloPoradi)) {
    pAkt=&((*pAkt)->Next);
    i++;
    }

  if(!(*pAkt)) {
    /* chteli bysme argument s poradivym cislem vetsim nez je delka seznamu
       tudiz doplnime volna mista volnymi promennymi */
    if(!((*pAkt)=new TPrvekHandlSeznamu)) m_error();    
    (*pAkt)->Info=VOLNA_PROMENNA;
    while(i<CisloPoradi) { 
      pAkt=&((*pAkt)->Next);
      if(!((*pAkt)=new TPrvekHandlSeznamu)) m_error();    
      (*pAkt)->Info=VOLNA_PROMENNA;
      i++;
      } /// of while()
    (*pAkt)->Next=0; ///ukoncime seznam  
    End=&((*pAkt)->Next); ///nastavime spravne posici End
    } ///of if()

  /* konecne nastavime hodnotu spravneho clenu seznamu */
  (*pAkt)->Info=InputArg;
  return 1;

} ///of nastav_arg()



int CHandlSeznam::vypust_arg(int CisloPoradi) {
/** vyznam: vypusti clen seznamu s poradovym cislem CisloPoradi
    vystup: 1 pri uspechu,
            0 jinak (zada se vyssi poradove cislo, nez je delka seznamu) */

TPrvekHandlSeznamu** ppAkt=&Head; /* ukazatel na ukazatel na aktualni prvek,
                                     prochazime jim seznam */
TPrvekHandlSeznamu* Del; ///ukazatel na ruseny prvek seznamu
int i=1; ///poradove cislo

  while((*ppAkt)&&(i<CisloPoradi)) {
    ppAkt=&((*ppAkt)->Next);
    i++;
    }

  if(!(*ppAkt)) 
    ///chteli bysme argument s poradivym cislem vestsim nez je delka seznamu
    return 0; 

  else { ///vse je OK
    Del=*ppAkt;
    (*ppAkt)=Del->Next;
    delete Del;
    /* musime pravit posici end, jsme-li na konci seznamu */
    if(!(*ppAkt))
      End=ppAkt;
    return 1;
    }

} ///of vypust_arg()



int CHandlSeznam::obsahuje(THandl Prvek) {
/** vyznam: otestuje, zda prvek Prvek se nachazi v seznamu,
            representovanem touto tridou;
            test na shodu se provadi 1stranne
            (Prvek musi byt obecnejsi ci roven nalezenemu)
    vystup: 1, obsahuje-li seznam Prvek
            0, neobsahuje-li */

TPrvekHandlSeznamu* Akt=Head; ///ukazatel na aktualni posoci v seznamu

  while(Akt) {
    if(db_shoda_ex((Akt->Info),Prvek) && (Akt->Info!=VOLNA_PROMENNA))
    ///Info muze byt zobecnene, ale VOLNA_PROMENNA znaci konec seznamu
      return 1;
    Akt=Akt->Next;
  }
  return 0; ///nenasli jsme
  
}///of obsahuje()



int CHandlSeznam::pocet() {
/** vyznam: vraci pocet clenu seznamu */

TPrvekHandlSeznamu* Akt=Head; ///ukazatel na aktualni posoci v seznamu
int i=0; ///aktualni pocet

  while(Akt) {
    i++;
    Akt=Akt->Next;
    }
  return i; 
  
}///of pocet()



void CHandlSeznam::clear() {
/** vyznam: vycisti (smaze) cely seznam, zbyde prazdny seznam */

  destroy(Head);
  Head=0; 
  End=&Head;
  PostupneCteni=Head;
  
} ///of clear()



void CHandlSeznam::nastav_zacatek_cteni(){
/** nastavi posici na postupne cteni seznamu na zacatek */

  PostupneCteni=Head;
} ///of nastav_zacatek_cteni()



int CHandlSeznam::nastav_zacatek_cteni(int Poradi){
/** vyznam: nastavi posici na postupne cteni seznamu
            na posici udanou v parametru Poradi
    vystup: 1, je-li vse OK
            0, je-li Poradi > delka seznamu */

int i=1; ///poradove cislo v seznamu

  PostupneCteni=Head;
  while(PostupneCteni && (i<Poradi)) {
    PostupneCteni=PostupneCteni->Next;
    i++;
    } ///of while
  return (!!(i==Poradi));

} ///of nastav_zacatek_cteni()



int CHandlSeznam::konec_cteni(){
/** vraci:<>0...nejsme na konci postupneho cteni CHandlSeznamu, 0...jsme */

  return (PostupneCteni==(*End));
}



THandl CHandlSeznam::precti_info(){
/** vyznam: precte hodnotu ulozenou v aktualnim (kam ukazuje PostupneCteni) 
            clenu seznamu a posune se o prvek dale
    vystup: prectena hodnota pri uspechu
            0 pri chybe */
	
THandl Obsah; ///obsah aktualniho clenu seznamu

  if(PostupneCteni){
    Obsah=PostupneCteni->Info;
    PostupneCteni=PostupneCteni->Next;
    return Obsah;
   }
  return 0;
 
} ///of precti_info()



THandl CHandlSeznam::nacti_info(){
/** vyznam: precte hodnotu ulozenou v aktulanim (kam ukazuje PostupneCteni) 
            clenu seznamu, ale neposune se dale 
    vystup: prectena hodnota pri uspechu
            0 pri chybe */
	
THandl Obsah; ///obsah aktualniho clenu seznamu

  if(PostupneCteni){
    Obsah=PostupneCteni->Info;
    return Obsah;
   }

  return 0;
 
} ///of nacti_info()



void CHandlSeznam::posun_dale(){
/** posune se na dalsi prvek seznamu (ukazatelem PostupneCteni) */
	
  if(PostupneCteni)
    PostupneCteni=PostupneCteni->Next;
 
} ///of precti_info()



void CHandlSeznam::vymaz_zbytek(){
/** vymaze vsechny prvky lezici za ukazatelem PostupneCteni */

TPrvekHandlSeznamu** ppAkt=&Head; /// ukazatel na ukazatel na aktualni prvek
	
  while((*ppAkt)!=PostupneCteni)
    ppAkt=&((*ppAkt)->Next);
  (*ppAkt)=0; ///ukoncime seznam
  destroy(PostupneCteni); ///odrizneme zbytek
  End=ppAkt;
 
} ///of vymaz_zbytek()



int CHandlSeznam::vymaz_zbytek(int Delka) {
/** vyznam: vymaze vsechny prvky lezici ve vzdalenosti vetsi nez Delka
    vystup: 1 */

TPrvekHandlSeznamu** ppAkt=&Head; /// ukazatel na ukazatel na aktualni prvek
TPrvekHandlSeznamu* Zbytek; /// zbytek sezanamu, ktery odrizneme

  while((Delka) &&((*ppAkt)!=0)) {
    ppAkt=&((*ppAkt)->Next);
    Delka--;
    } ///of while()
  if(Delka) ///seznam je kratsi nez Delka, nic se neudela
    return 1;
  Zbytek=(*ppAkt);
  (*ppAkt)=0; ///ukoncime seznam
  destroy(Zbytek); ///odrizneme zbytek
  End=ppAkt;
  return 1;

} ///of vymaz_zbytek()



THandl* CHandlSeznam::precti_p_info(){
/** vystup: vraci ukazatel na hodnotu ulozenou v aktualnim
            (kam ukazuje PostupneCteni) clenu seznamu 
            0 pri chybe */

THandl* pObsah; ///ukazatel na obsah aktualniho clenu seznamu
	
  if(PostupneCteni){
    pObsah=&(PostupneCteni->Info);
    PostupneCteni=PostupneCteni->Next;
    return pObsah;
   }

  return 0;

} ///of precti_p_info()



int CHandlSeznam::kopie_seznamu(CHandlSeznam& SezParam, int DelkaOpakovani) {
/** vyznam: prekopiruje ze SezParam (od posice ukazatele PostupneCteni)
            do naseho seznamu na konec seznam logicky opakujicich se 
            polozek o poctu DelkaOpakovani
            a seznam je ukonceny polozkou VOLNA_PROMENNA (zkopiruje i ji!)
    vystup: 1, je-li vse OK
            0 pri chybe - seznam v SezParam neni ukonceny VOLNA_PROMENNA */

THandl Prvek; ///konkretni clen seznamu, ktery kopirujeme
int Delka=0; ///posice uvnitr jednoho cyklu logicky opakujicich se polozek

  while(!SezParam.konec_cteni()) {
    Prvek=SezParam.precti_info();
    pridej_na_konec(Prvek);
    if( (Prvek==VOLNA_PROMENNA) && (!Delka))
      return 1; ///jsme na konci
    Delka=(Delka+1)%DelkaOpakovani;
    } ///of while
  return 0; ///spatny vstup

} ///of kopie_seznamu()



int CHandlSeznam::kopie_seznamu(CHandlSeznam& SezParam) {
/** vyznam: prekopiruje ze SezParam do naseho seznamu na konec seznam vsech 
            prvku od posice ukazatele PostupneCteni do konce
    vystup: 1, je-li vse OK */

THandl Prvek; ///konkretni clen seznamu, ktery kopirujeme

  while(!SezParam.konec_cteni()) {
    Prvek=SezParam.precti_info();
    pridej_na_konec(Prvek);
    } ///of while
  return 1; 

} ///of kopie_seznamu()



TPrvekHandlSeznamu* CHandlSeznam::fill(const TPrvekHandlSeznamu* NovySeznam){
/** vyznam: okopiruje seznam NovySeznam do noveho CHandlSeznamu 
    vraci: ukazatel na novy seznam */

TPrvekHandlSeznamu* pZbytek; ///ukazatel na zbytek seznamu za 1. clenem
TPrvekHandlSeznamu* pPrvni; ///ukazatel na 1. clen seznamu

  if(NovySeznam){
    pZbytek=fill(NovySeznam->Next);
    if(!(pPrvni=new TPrvekHandlSeznamu)) m_error();
    pPrvni->Info=NovySeznam->Info;
    pPrvni->Next=pZbytek;
    return pPrvni;
    }
  else return 0;
 
} ///of fill()



void CHandlSeznam::nastav_posici_end() { 
/** nastavi ukazatel End na konec seznamu */

  End=&Head; 
  while(*End) End=&((*End)->Next);

} ///of nastav_posici_end()



void CHandlSeznam::destroy(TPrvekHandlSeznamu* pPrvekSeznamu){
/** uvolni dynamicky alokovanou pamet seznamu - zrusi vsechny prvky */

TPrvekHandlSeznamu* pPrvni; ///ukazatel na 1. clem seznamu
	
  while(pPrvekSeznamu){
    pPrvni=pPrvekSeznamu;
    pPrvekSeznamu=pPrvekSeznamu->Next;
    delete pPrvni;
    }
 
} ///of destroy()

