/********************************************************
* hlavickovy soubor: SEZNAM.H
* autor: Milan Hladik
* verse: 1.3.
* datum: 24.5.2002
*********************************************************
* obsah:
*   - deklarace tridy CHandlSeznam,
*     representujici spojovy seznam s prvky typu THandl
*   - obsahuje metody pro operace se spojovym seznamem:
*     pridavani (na zacatek, konec, pripojeni), vymazani,
*     cteni (postupne, z libovolne posice),... 
*********************************************************/


#ifndef _handl_seznam_h_
#define _handl_seznam_h_


#include "extra.h"  
///zatim pro osobni ucely - definice THandl a pod.


typedef struct _PrvekHandlSeznamu {
/** struktura pro prvek seznamu pro tridu CHandlSeznam */

  THandl Info; ///vlastni hodnota prvku seznamu
  struct _PrvekHandlSeznamu* Next; ///ukazatel na dalsi prvek seznamu

} TPrvekHandlSeznamu;



/************************************************************
*  trida CHandlSeznam 
*************************************************************/

/** deklarace tridy representujici spojovy seznam s vkladanim a ctenim
    prvky seznamu jsou hodnoty typu THandl
    trida se pouziva jako seznam parametru pro smysly (viz df. TSmysl) */

class CHandlSeznam {

public:
  CHandlSeznam();  ///implicitni konstruktor
  CHandlSeznam(const CHandlSeznam& NovySeznam);  ///kopirovaci konstruktor
  ~CHandlSeznam();  ///destruktor

  const CHandlSeznam& operator=(const CHandlSeznam& NovySeznam);
  /* operator prirazeni */
	
  void pridej_na_zacatek(THandl NoveInfo); ///prida novy clen seznamu na zacatek
  void pridej_na_konec(THandl NoveInfo);  ///prida novy clen seznamu na konec

  int precti_arg(int CisloPoradi, THandl* OutputArg);
  /** vyznam: precte hodnotu clenu seznamu s poradovym cislem CisloPoradi
              a vrati ji v parametru OutputArg
      vystup: 1 pri uspechu,
              0 jinak (zada se vyssi poradove cislo, nez je delka seznamu) */  

  int nastav_arg(int CisloPoradi, THandl InputArg);
  /** vyznam: nastavi hodnotu clenu seznamu s poradovym cislem CisloPoradi
              na hodnotu InputArg
      vystup: 1 pri uspechu,
              0 jinak (zada-li se poradove cislo zaporne ci 0) */

  int vypust_arg(int CisloPoradi);
  /** vyznam: vypusti clen seznamu s poradovym cislem CisloPoradi
      vystup: 1 pri uspechu,
              0 jinak (zada se vyssi poradove cislo, nez je delka seznamu) */
 
  int obsahuje(THandl Prvek);
  /** vyznam: otestuje, zda prvek Prvek se nachazi v seznamu,
              representovanem touto tridou
              test na shodu se provadi 1stranne
              (Prvek musi byt obecnejsi ci roven nalezenemu)
      vystup: 1, obsahuje-li seznam Prvek
              0, neobsahuje-li */

  int pocet(); 
  /** vyznam: vraci pocet clenu seznamu */

  void clear();
  /** vyznam: vycisti (smaze) cely seznam, zbyde prazdny seznam */

  void nastav_zacatek_cteni();
  /* nastavi posici na postupne cteni seznamu na zacatek */

  int nastav_zacatek_cteni(int Poradi);
  /** vyznam: nastavi posici na postupne cteni seznamu
              na posici udanou v parametru Poradi
      vystup: 1, je-li vse OK
              0, je-li Poradi > delka seznamu */

  int konec_cteni();  ///nejsme na konci postupneho cteni seznamu

  THandl precti_info();
  /** vyznam: precte hodnotu ulozenou v aktualnim clenu seznamu
              a posune se na dalsi clen 
      vystup: prectena hodnota pri uspechu
              0 pri chybe */

  THandl nacti_info();
  /** vyznam: precte hodnotu ulozenou v aktualnim (kam ukazuje PostupneCteni) 
              clenu seznamu, ale neposune se dale 
      vystup: prectena hodnota pri uspechu
              0 pri chybe */

  void posun_dale();
  /** posune se na dalsi prvek seznamu (ukazatelem PostupneCteni) */

  void vymaz_zbytek();
  /** vymaze vsechny prvky lezici za ukazatelem PostupneCteni */

  int vymaz_zbytek(int Delka);
  /** vymaze vsechny prvky lezici ve vzdalenosti vetsi nez Delka */

  THandl* precti_p_info();
  /** vystup: vraci ukazatel na hodnotu ulozenou v aktualnim clenu seznamu
              a posune se na dalsi clen
              0 pri chybe */

  int kopie_seznamu(CHandlSeznam& SezParam, int DelkaOpakovani);
  /** vyznam: prekopiruje ze SezParam (od ukazatele PostupneCteni) do naseho
              seznamu na konec seznam logicky
              opakujicich se polozek o poctu DelkaOpakovani
              a seznam je ukonceny polozkou VOLNA_PROMENNA (zkopiruje i ji)
      vystup: 1, je-li vse OK
              0 pri chybe - seznam v SezParam neni ukonceny VOLNA_PROMENNA */

  int kopie_seznamu(CHandlSeznam& SezParam);
  /** vyznam: prekopiruje ze SezParam do naseho seznamu na konec seznam vsech 
              prvku od posice ukazatele PostupneCteni do konce
      vystup: 1, je-li vse OK */

protected:
  TPrvekHandlSeznamu* Head;  ///ukazatel na zacatek seznamu

  TPrvekHandlSeznamu** End;  
  /* ukazatel na ukazatel Next posledniho prvku seznamu,
     jinak ukazatel na Head, je-li seznam prazdny*/

  TPrvekHandlSeznamu* PostupneCteni; 
  /* aktualni clen seznamu pro postupne cteni */
		
  TPrvekHandlSeznamu* fill(const TPrvekHandlSeznamu* NovySeznam); 
  /* okopiruje NovySeznam do noveho seznamu */

  void nastav_posici_end(); ///nastavi ukazatel End na konec seznamu
  void destroy(TPrvekHandlSeznamu* pPrvekSeznamu);  ///uvolni pamet ze seznamu
 };


#endif ///of _handl_seznam_h_

